/**
 * Copyright (c) Members of the EGEE Collaboration. 2006-2010.
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors:
 * 2009-
 *	Oscar Koeroo <okoeroo@nikhef.nl>
 *	Mischa Sall\'e <msalle@nikhef.nl>
 *	NIKHEF Amsterdam, the Netherlands
 *
 *  2006-2009
 *	Gerben Venekamp <venekamp@nikhef.nl>
 *	NIKHEF Amsterdam, the Netherlands
 *
 */

/*!
 *  \file glexec.h
 *
 *  Provides global defines for gLExec
 *
 */

#ifndef _GLEXEC_H
#define _GLEXEC_H

/* Defines coming for configure: e.g PACKAGE and VERSION */
#include "glexec_config.h"

/* Need sys/types.h for uid_t and gid_t */
#include <sys/types.h>

/************************************************************************/
/* USER DEFINES	                                                        */
/************************************************************************/

/* The default glexec.conf file and location GLEXEC_CONFIG_FILE is set in configure. */

/* GLEXEC_WHITELIST_GROUP: members of this group are automatically whitelisted */
#ifndef GLEXEC_WHITELIST_GROUP
    #define GLEXEC_WHITELIST_GROUP "glexec"
#endif

/* GLEXEC_USER: when reading the configuration file, privileges are dropped to
 * this user and group GLEXEC_CONFIG_GROUP, furthermore, glexec.conf is allowed
 * to be owned by root or this user. Furthermore, this user may run glexec -V */
#ifndef GLEXEC_USER
    #define GLEXEC_USER "glexec"
#endif

/* Reading of config file is done with this unix group */
#ifndef GLEXEC_CONFIG_GROUP
    #define GLEXEC_CONFIG_GROUP "glexec"
#endif

/* Used to define a default for the TARGET_PROXY, using mkstemp(), %u will be
 * substituted for the uid */
#ifndef GLEXEC_DEFAULT_NAME_TARGET_PROXY
    #define GLEXEC_DEFAULT_NAME_TARGET_PROXY "/tmp/x509up_u%u.glexec.XXXXXX"
#endif

/* Define a safe PATH environment to pass to executables. */
#ifndef GLEXEC_TARGET_PATH
    #define GLEXEC_TARGET_PATH "/usr/local/bin:/usr/bin:/bin"
#endif

/* Define umask for the target process (also used to write the target proxy). */
#ifndef GLEXEC_TARGET_UMASK
    #define GLEXEC_TARGET_UMASK ~((mode_t)S_IRWXU)
#endif

/* Define internal umask */
#ifndef GLEXEC_RUNTIME_UMASK
    #define GLEXEC_RUNTIME_UMASK ~((mode_t)S_IRWXU)
#endif

/* UID_MIN -- Define this as the lowest UID allowed to be a target user
 *            for  gLExec.  For most systems, 500 or 100 is common, but
 *            99 will include user nobody on RedHat Linux systems. */
#ifndef GLEXEC_UID_MIN
    #define GLEXEC_UID_MIN 99U
#endif

/* GID_MIN -- Define this as the lowest GID allowed to be a target group
 *            for gLExec.  For most systems, 100 is common, but 99 will
 *            include group nobody on RedHat Linux systems. */
#ifndef GLEXEC_GID_MIN
    #define GLEXEC_GID_MIN 99U
#endif

/* Define maximum number of printed characters in argument list before
 * truncating */
#define GLEXEC_MAX_PRINT_ARGLIST    100U

/************************************************************************
 * Handle with care! 
 *
 * Enabling the following define will DISABLE various SECURITY MEASURES that
 * will result in an UNSAFE gLExec 
 ************************************************************************/
/* #define YES_I_AM_REALLY_SURE_TO_DISABLE_THIS_SECURITY_MEASURE_IN_GLEXEC */
/************************************************************************/

/************************************************************************/
/* INTERNAL DEFINES	                                                */
/************************************************************************/

/* PACKAGE comes from configure.ac */
#ifndef GLEXEC_PACKAGE_NAME
    #define GLEXEC_PACKAGE_NAME PACKAGE
#endif

/* VERSION comes from configure.ac */
#ifndef GLEXEC_VERSION
    #define GLEXEC_VERSION VERSION
#endif

/* Not all platforms have PATH_MAX, make a suitable default */
#if defined(PATH_MAX)
 #define AP_MAXPATH PATH_MAX
#elif defined(MAXPATHLEN)
 #define AP_MAXPATH MAXPATHLEN
#else
 #define AP_MAXPATH 4096
#endif

/************************************************************************/
/* TYPES								*/
/************************************************************************/

/* Different locking mechanisms */
typedef enum GLEXEC_LOCK_MECH_E
{
    GLEXEC_LOCK_FLOCK     = 1,
    GLEXEC_LOCK_FCNTL     = 2,
    GLEXEC_LOCK_DISABLED  = 3,
    GLEXEC_LOCK_UNKNOWN	  = 4
} GLEXEC_LOCK_MECH_T;

/* Struct containing user information for caller, target etc. */
typedef struct glexec_account_info_s	{
    uid_t uid;
    gid_t gid;	    /* primary gid */
    gid_t *sgid;   /* secondary gids */
    int nsgid;	    /* number of sec. gids */
    char *uname;
    char *gname;    /* primary groupname */
    char *dir;
} glexec_account_t;


#endif /* _GLEXEC_H */
